import logging
from pathlib import Path
from typing import Optional, List
from typing_extensions import Annotated
import inspect

import typer

from rsyncrun.config import Config
from rsyncrun.rsync import invoke_rsync

app = typer.Typer()

logger = logging.getLogger("rsync_runner")

LOG_LEVELS = ["DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"]


def get_kwargs():
    frame = inspect.currentframe().f_back
    keys, _, _, values = inspect.getargvalues(frame)
    kwargs = {}
    for key in keys:
        if key != 'self':
            kwargs[key] = values[key]
    return kwargs


@app.command()
def main(
        source_dir: Annotated[str, typer.Argument()],
        dest_dir: Annotated[str, typer.Argument()],
        exclude_from: Annotated[Optional[List[Path]], typer.Option(file_okay=True, dir_okay=False)] = None,
        include_from: Annotated[Optional[List[Path]], typer.Option(file_okay=True, dir_okay=False)] = None,
        create_dirs: Annotated[bool, typer.Option()] = True,
        log_path: Annotated[Optional[Path], typer.Option(file_okay=True, dir_okay=False)] = None,
        log_level: Annotated[str, typer.Option()] = None,
        dry_run: Annotated[bool, typer.Option()] = False,
        write_report: Annotated[bool, typer.Option()] = False
):
    kwargs = get_kwargs()
    config_kwargs = dict(env_prefix="rsyncrunner")
    config_kwargs.update(**kwargs)
    config = Config(**config_kwargs)
    if log_level and log_level not in LOG_LEVELS:
        raise ValueError("Invalid log level, use one of ({', '.join(LOG_LEVELS)})")

    setup_logging_kwargs = dict()
    if log_path:
        setup_logging_kwargs['log_path'] = str(log_path)
    if log_level:
        setup_logging_kwargs['log_level'] = log_level
    config.setup_logging()

    logger.info(f"Starting backup from {str(Path(source_dir).resolve())} to {str(Path(dest_dir).resolve())}")
    logger.info(f"Log directory path is {config.log_directory_path}")

    try:
        invoke_rsync(config)
    except Exception as e:
        logger.error(e)


if __name__ == "__main__":
    typer.run(main)
